@@ -1,4 +1,4 @@
-## Copyright (C) 2009,2010,2011,2012 Philip Nienhuis <prnienhuis at users.sf.net>
+## Copyright (C) 2009,2010,2011,2012,2013 Philip Nienhuis
 ##
 ## This program is free software; you can redistribute it and/or modify it under
 ## the terms of the GNU General Public License as published by the Free Software
@@ -17,7 +17,7 @@
 ## @deftypefn {Function File} [@var{ods}] = odsclose (@var{ods})
 ## @deftypefnx {Function File} [@var{ods}] = odsclose (@var{ods}, @var{filename})
 ## @deftypefnx {Function File} [@var{ods}] = odsclose (@var{ods}, "FORCE")
-## Close the OpenOffice_org Calc spreadsheet pointed to in struct
+## Close the OpenOffice_org Calc / Gnumeric spreadsheet pointed to in struct
 ## @var{ods}, if needed write the file to disk.
 ## odsclose will determine if the file must be written to disk based on
 ## information contained in @var{ods}.
@@ -28,10 +28,10 @@
 ## file pointer struct. However, in case of UNO, a hidden OOo invocation
 ## may linger on in memory then, preventing proper closing of Octave.
 ##
-## You need the Java package >= 1.2.6 plus odfdom.jar + xercesImpl.jar
-## and/or jopendocument-<version>.jar installed on your computer +
-## proper javaclasspath set, to make this function work at all.
-## For UNO support, Octave-Java package >= 1.2.8 + latest fixes is imperative;
+## For writing .ods files you need the Java package >= 1.2.6 plus odfdom.jar
+## + xercesImpl.jar and/or jopendocument-<version>.jar installed on your
+## computer + proper javaclasspath set, to make this function work at all.
+## For UNO support, Octave-Java package >= 1.2.9 is imperative;
 ## furthermore the relevant classes had best be added to the javaclasspath by
 ## utility function chk_spreadsheet_support().
 ##
@@ -49,7 +49,7 @@
 ##
 ## @end deftypefn
 
-## Author: Philip Nienhuis
+## Author: Philip Nienhuis <prnienhuis at users.sf.net>
 ## Created: 2009-12-13
 ## Updates:
 ## 2010-01-08 (OTK ODS write support)
@@ -70,6 +70,10 @@
 ##     ''     Move "file ptr preserved" message to proper else clause
 ## 2012-10-23 Style fixes
 ## 2012-12-18 Improved error/warning messages
+## 2013-09-09 Native Octave interface ("OCT") for reading
+##      ''    Warning message for empty file ptr structs
+## 2013-10-02 Texinfo header adapted
+## 2013-12-01 Style fixes
 
 function [ ods ] = odsclose (ods, varargs)
 
@@ -80,17 +84,22 @@ function [ ods ] = odsclose (ods, varargs)
 
   force = 0;
 
+  if (isempty (ods))
+    warning ("odsclose: file pointer struct is empty; was it already closed?")';
+    return
+  endif
+
   if (nargin > 1)
     for ii=2:nargin
       if (strcmpi(varargin{ii}, "force"))
         ## Close .ods anyway even if write errors occur
         force = 1;
 
-      elseif (~isempty (strfind (tolower (varargin{ii}), ".")))
+      elseif (! isempty (strfind (tolower (varargin{ii}), ".")))
         ## Apparently a file name. First some checks....
         if (ods.changed == 0 || ods.changed > 2)
           warning ("odsclose.m: file %s wasn't changed, new filename ignored.", ods.filename);
-        elseif (~strcmp (xls.xtype, "UNO") && ...
+        elseif (! strcmp (xls.xtype, "UNO") && ...
                 isempty (strfind ( lower (filename), ".ods")))
           ## UNO will write any file type, all other interfaces only .ods
             error ("odsclose.m: .ods suffix lacking in filename %s", filename);
@@ -115,6 +124,10 @@ function [ ods ] = odsclose (ods, varargs)
     ## Java & UNO bridge
     ods = __UNO_spsh_close__ (ods, force);
 
+  elseif (strcmp (ods.xtype, "OCT"))
+    ## Native Octave
+    ods = __OCT_spsh_close__ (ods);
+
   ##elseif ---- < Other interfaces here >
 
   else
