@@ -1,4 +1,4 @@
-## Copyright (C) 2009,2010,2011,2012 Philip Nienhuis <prnienhuis at users.sf.net>
+## Copyright (C) 2009,2010,2011,2012,2013 Philip Nienhuis
 ##
 ## This program is free software; you can redistribute it and/or modify it under
 ## the terms of the GNU General Public License as published by the Free Software
@@ -37,9 +37,9 @@
 ## spreadsheet has to be parsed to get the sheet names, let alone exploring
 ## used data ranges.
 ##
-## By specifying a value of 'jod', 'otk' or 'uno' for @var{reqintf} the automatic
-## selection of the java interface is bypassed and the specified interface
-## will be used (if at all present).
+## By specifying a value of 'jod', 'otk', 'uno' or 'oct' for @var{reqintf} the
+## automatic selection of the java interface is bypassed and the specified
+## interface will be used (if at all present).
 ##
 ## Examples:
 ##
@@ -79,6 +79,9 @@
 ## 2012-06-08 Support for odfdom-0.8.8-incubator
 ## 2012-10-12 Moved all interface-specific code into ./private subfuncs
 ## 2012-10-24 Style fixes
+## 2013-09-09 Native Octave interface ("OCT") for reading
+## 2013-09-23 Header updated to OCT interface
+## 2013-12-01 Output arg #2 adapted to file type tru switch stmt
 
 function [ filetype, sh_names ] = odsfinfo (filename, reqintf=[])
 
@@ -91,7 +94,16 @@ function [ filetype, sh_names ] = odsfinfo (filename, reqintf=[])
   ## If no ods support was found, odsopen will have complained. Just return here
   if (isempty (ods)), return; endif
   
-  filetype = "OpenOffice.org Calc Document";
+  ## If any valid xls-pointer struct has been returned, it must be a valid
+  ## spreadsheet. Find out what format
+  [~, ~, ext] = fileparts (ods.filename); 
+  switch ext
+    case {"ods", "sxc"}
+      filetype = "OpenOffice.org Calc Document";
+    case "gnumeric"
+      filetype = "Gnumeric spreadsheet";
+    otherwise
+  endswitch
 
   ## To save execution time, only proceed if sheet names are wanted
   if ~(nargout == 1)
@@ -105,6 +117,9 @@ function [ filetype, sh_names ] = odsfinfo (filename, reqintf=[])
     elseif (strcmp (ods.xtype, "UNO"))
       [sh_names] = __UNO_spsh_info__ (ods);
 
+    elseif (strcmp (ods.xtype, "OCT"))
+      [sh_names] = __OCT_spsh_info__ (ods);
+
     else
       ## Below error will have been catched in odsopen() above
       ##error (sprintf ("odsfinfo: unknown OpenOffice.org .ods interface - %s.",...
