@@ -1,4 +1,4 @@
-## Copyright (C) 2009,2010,2011,2012,2013 Philip Nienhuis <prnienhuis at users.sf.net>
+## Copyright (C) 2009,2010,2011,2012,2013,2014 Philip Nienhuis
 ## 
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -23,14 +23,17 @@
 ##
 ## Calling odsopen without specifying a return argument is fairly useless!
 ##
-## To make this function work at all, you need the Java package >= 1.2.5 plus
-## ODFtoolkit (version 0.7.5 or 0.8.6+) & xercesImpl, and/or jOpenDocument, and/or
-## OpenOffice.org (or clones) installed on your computer + proper javaclasspath
-## set. These interfaces are referred to as OTK, JOD, and UNO resp., and are
-## preferred in that order by default (depending on their presence).
-## For (currently experimental) UNO support, Octave-Java package 1.2.8 + latest
-## fixes is imperative; furthermore the relevant classes had best be added to
-## the javaclasspath by utility function chk_spreadsheet_support().
+## Octave links to external software for read/write support of spreadsheets;
+## these links are "interfaces". For just reading ODS 1.2 and Gnumeric XML
+## no external SW is required, yet this "interface" is called 'OCT'.
+## To make this function work at all for write support, you need a Java JRE
+## or JDK plus one or more of (ODFtoolkit (version 0.7.5 or 0.8.6 - 0.8.8) &
+## xercesImpl v.2.9.1), jOpenDocument, or OpenOffice.org (or clones) installed
+## on your computer + proper javaclasspath set. These interfaces are referred
+## to as OTK, JOD, and UNO resp., and are preferred in that order by default
+## (depending on their presence; the OCT interface has lowest priority).
+## The relevant Java class libs for spreadsheet I/O had best be added to the
+## javaclasspath by utility function chk_spreadsheet_support().
 ##
 ## @var{filename} must be a valid .ods OpenOffice.org file name including
 ## .ods suffix. If @var{filename} does not contain any directory path,
@@ -43,10 +46,10 @@
 ## is desired immediately after calling odsopen(). It merely serves proper
 ## handling of file errors (e.g., "file not found" or "new file created").
 ##
-## Optional input argument @var{reqintf} (character string, or cellstr array)
-## can be used to override the ODS interface automatically selected by odsopen.
-## Currently implemented interfaces are 'OTK' (Java/ODF Toolkit),
-## 'JOD' (Java/jOpenDocument) and 'UNO' (Java/OpenOffice.org UNO bridge).
+## Optional input argument @var{reqintf} can be used to override the ODS
+## interface automatically selected by odsopen. Currently implemented interfaces
+## are 'OTK' (Java/ODF Toolkit), 'JOD' (Java/jOpenDocument), 'UNO'
+## (Java/OpenOffice.org UNO bridge), and 'OCT' (native Octave, only reading).
 ## In most situations this parameter is unneeded as odsopen automatically
 ## selects the most useful interface present ("default interface").
 ## Depending on file type, odsopen.m can invoke other detected interfaces than
@@ -71,7 +74,7 @@
 ##
 ## @end deftypefn
 
-## Author: Philip Nienhuis
+## Author: Philip Nienhuis <prnienhuis at users.sf.net>
 ## Created: 2009-12-13
 ## Updates: 
 ## 2009-12-30 ....<forgot what is was >
@@ -117,54 +120,75 @@
 ##      ''    Removed fall-back options for .sxc. Other than .xls this can be
 ##            inferred from file suffix
 ## 2012-12-18 Improved error messages
-## 2012-03-01 Revamped logic for determining if file type is supported by a
-##            particular interface
+## 2013-02-24 Temporarily disabled UNO
+## 2013-06-18 Re-enabled UNO
+## 2013-09-09 Native Octave interface ("OCT") for reading
+## 2013-09-23 Check and catch write requests for OCT interface
+##     ''     Relax check for lowercase filename extension
+## 2013-09-29 Initially set OCT interface to not detected
+## 2013-11-04 Revert above
+## 2013-11-08 Better filetype / file extension detection (bug #40490)
+##     ''     Removed stray ';'
+## 2013-11-12 Fix syntax error + missing initialization in file ext switch stmt
+## 2013-12-01 Updated texinfo header
+## 2013-12-18 Style fixes
+## 2013-12-27 Use one variable for processed file types
+## 2014-01-01 Add warning that UNO will write ODS f. unsupported file extensions
+##     ''     Copyright string update
 
 function [ ods ] = odsopen (filename, rw=0, reqinterface=[])
 
   persistent odsinterfaces; persistent chkintf; persistent lastintf;
   if (isempty (chkintf))
-    odsinterfaces = struct ( "OTK", [], "JOD", [], "UNO", [] );
+    odsinterfaces = struct ( "OTK", [], "JOD", [], "UNO", [] , "OCT", 1);
     chkintf = 1;
   endif
-  if (isempty (lastintf)); lastintf = "---"; endif 
+  if (isempty (lastintf));
+    lastintf = "---";
+  endif
+  odsintf_cnt = 1;
   
   if (nargout < 1)
     usage ("ODS = odsopen (ODSfile, [Rw]). But no return argument specified!");
   endif
 
-  if (~isempty (reqinterface))
-    if ~(ischar (reqinterface) || iscell (reqinterface))
+  if (! isempty (reqinterface))
+    if (! (ischar (reqinterface) || iscell (reqinterface)))
       usage ("odsopen.m: arg # 3 (interface) not recognized");
     endif
     ## Turn arg3 into cell array if needed
-    if (~iscell (reqinterface)), reqinterface = {reqinterface}; endif
+    if (! iscell (reqinterface))
+      reqinterface = {reqinterface};
+    endif
     ## Check if previously used interface matches a requested interface
     if (isempty (regexpi (reqinterface, lastintf, "once"){1}))
-      ## New interface requested
+      ## New interface requested. OCT is always supported but it must be
+      ## possible to disable it
       odsinterfaces.OTK = 0; odsinterfaces.JOD = 0; odsinterfaces.UNO = 0;
+      odsinterfaces.OCT = 0;
       for ii=1:numel (reqinterface)
         reqintf = toupper (reqinterface {ii});
         ## Try to invoke requested interface(s) for this call. Check if it
         ## is supported anyway by emptying the corresponding var.
-        if     (strcmp (reqintf, "OTK"))
+        if     (strcmpi (reqintf, "OTK"))
           odsinterfaces.OTK = [];
-        elseif (strcmp (reqintf, "JOD"))
+        elseif (strcmpi (reqintf, "JOD"))
           odsinterfaces.JOD = [];
-        elseif (strcmp (reqintf, "UNO"))
+        elseif (strcmpi (reqintf, "UNO"))
           odsinterfaces.UNO = [];
+        elseif (strcmpi (reqintf, "OCT"))
+          odsinterfaces.OCT = [];
         else 
           usage (sprintf (["odsopen.m: unknown .ods interface \"%s\" requested.\n" ...
-                  "Only OTK, JOD or UNO supported\n"], reqinterface{}));
+                  "Only OTK, JOD, UNO and OCT supported\n"], reqinterface{}));
         endif
       endfor
 
       printf ("Checking requested interface(s):\n");
       odsinterfaces = getodsinterfaces (odsinterfaces);
       ## Well, is/are the requested interface(s) supported on the system?
-      odsintf_cnt = 0;
       for ii=1:numel (reqinterface)
-        if (~odsinterfaces.(toupper (reqinterface{ii})))
+        if (! odsinterfaces.(toupper (reqinterface{ii})))
           ## No it aint
           printf ("%s is not supported.\n", toupper (reqinterface{ii}));
         else
@@ -172,7 +196,7 @@ function [ ods ] = odsopen (filename, rw=0, reqinterface=[])
         endif
       endfor
       ## Reset interface check indicator if no requested support found
-      if (~odsintf_cnt)
+      if (! odsintf_cnt)
         chkintf = [];
         ods = [];
         return
@@ -183,13 +207,23 @@ function [ ods ] = odsopen (filename, rw=0, reqinterface=[])
   ## Var rw is really used to avoid creating files when wanting to read, or
   ## not finding not-yet-existing files when wanting to write a new one.
   ## Be sure it's either 0 or 1 initially
-  if (rw), rw = 1; endif      
+  if (rw)
+    if (odsintf_cnt == 1 && odsinterfaces.OCT)
+      ## Check if OCT is only interface and writing is requested
+      error ("OCT interface doesn't support writing files");
+    endif
+    rw = 1;
+  endif
 
   ## Check if ODS file exists. Set open mode based on rw argument
-  if (rw), fmode = "r+b"; else fmode = "rb"; endif
+  if (rw)
+    fmode = "r+b";
+  else
+    fmode = "rb";
+  endif
   fid = fopen (filename, fmode);
   if (fid < 0)
-    if (~rw)                  ## Read mode requested but file doesn't exist
+    if (! rw)                 ## Read mode requested but file doesn't exist
       err_str = sprintf ("odsopen.m: file %s not found\n", filename);
       error (err_str)
     else        
@@ -216,10 +250,17 @@ function [ ods ] = odsopen (filename, rw=0, reqinterface=[])
   [odsinterfaces] = getodsinterfaces (odsinterfaces);
 
   ## Supported interfaces determined; now check ODS file type.
-
-  chk3 = strcmpi (filename(end-3:end), '.ods');
-  ## Only jOpenDocument (JOD) can read from .sxc files, but only if odfvsn = 2
-  chk4 = strcmpi (filename(end-3:end), '.sxc');
+  ftype = 0;
+  [~, ~, ext] = fileparts (filename);
+  switch ext
+    case ".ods"               ## ODS 1.2
+      ftype = 3;
+    case ".sxc"               ## jOpenDocument (JOD) can read from .sxc files,
+      ftype = 4;               ## but only if odfvsn = 2
+    case ".gnumeric"          ## Zipped XML / gnumeric
+      ftype = 5;
+    otherwise
+  endswitch
 
   ods = struct ("xtype",    [], 
                 "app",      [], 
@@ -230,37 +271,43 @@ function [ ods ] = odsopen (filename, rw=0, reqinterface=[])
                 "odfvsn",   []);
 
   ## Preferred interface = OTK (ODS toolkit & xerces), so it comes first. 
-  ## Keep track of which interface is selected. Can be used for fallback to other intf
+  ## Keep track of which interface is selected. Can be used (later) for
+  ## fallback to other interface
   odssupport = 0;
 
-  if (odsinterfaces.OTK && ~odssupport && chk3)
+  if (odsinterfaces.OTK && ! odssupport && ftype == 3)
     [ ods, odssupport, lastintf ] = ...
               __OTK_spsh_open__ (ods, rw, filename, odssupport);
   endif
 
-  if (odsinterfaces.JOD && ~odssupport && (chk3 || chk4))
+  if (odsinterfaces.JOD && ! odssupport && (ftype == 3 || ftype == 4))
     [ ods, odssupport, lastintf ] = ...
               __JOD_spsh_open__ (ods, rw, filename, odssupport);
   endif
 
-  if (odsinterfaces.UNO && ~odssupport)
+  if (odsinterfaces.UNO && ! odssupport && ftype < 5)
+    ## Warn for LO / OOo stubbornness
+    if (ftype == 0 || ftype == 5 || ftype == 6)
+      warning ("UNO interface will write ODS format for unsupported file extensions")
+    endif
     [ ods, odssupport, lastintf ] = ...
               __UNO_spsh_open__ (ods, rw, filename, odssupport);
   endif
 
+  if (odsinterfaces.OCT && ! odssupport && ...
+      (ftype == 2 || ftype == 3 || ftype == 5))
+    [ ods, odssupport, lastintf ] = ...
+              __OCT_spsh_open__ (ods, rw, filename, odssupport, ftype);
+  endif
+
   ## if 
-  ##   <other future interfaces here>
+  ##   <other interfaces here>
 
   if (! odssupport)
-    if (isempty (reqinterface))
-      ## Below message follows after getodsinterfaces found no support
-      printf ("None.\n");
-      warning ("odsopen.m: no support for OpenOffice.org .ods I/O"); 
-    else
-      warning ("odsopen.m: file type not supported by %s %s %s", upper (reqinterface{:}));
-    endif
+    ## Below message follows after getodsinterfaces
+    printf ("None.\n");
+    warning ("odsopen.m: no support for OpenOffice.org .ods I/O"); 
     ods = [];
-    ## Reset found interfaces for re-testing in the next call. Add interfaces if needed.
     chkintf = [];
   else
     # From here on rw is tracked via ods.changed in the various lower
@@ -271,7 +318,9 @@ function [ ods ] = odsopen (filename, rw=0, reqinterface=[])
     # ods.changed = 0 (existing/only read from), 1 (existing/data added), 2 (new,
     # data added) or 3 (pristine, no data added).
     # Until something was written to existing files we keep status "unchanged".
-    if (ods.changed == 1); ods.changed = 0; endif
+    if (ods.changed == 1);
+      ods.changed = 0;
+    endif
   endif
 
 endfunction
