@@ -1,4 +1,4 @@
-## Copyright (C) 2009,2010,2011,2012 Philip Nienhuis <prnienhuis at users.sf.net>
+## Copyright (C) 2009,2010,2011,2012,2013 Philip Nienhuis <prnienhuis at users.sf.net>
 ##
 ## This program is free software; you can redistribute it and/or modify it under
 ## the terms of the GNU General Public License as published by the Free Software
@@ -78,9 +78,18 @@
 ##     ''     Move "file ptr preserved" message to proper else clause
 ## 2012-10-24 Style fixes
 ## 2012-12-18 Improved error/warning messages
+## 2013-09-30 OCT interface added
+## 2013-10-01 Warn for empty struct input
+## 2013-12-29 Style fixes
+## 2014-01-01 Style fixes
 
 function [ xls ] = xlsclose (xls, varargs)
 
+  if (isempty (xls))
+    warning ("xlsclose: file pointer struct was already closed");
+    return
+  endif
+
   force = 0;
 
   if (nargin > 1)
@@ -90,13 +99,13 @@ function [ xls ] = xlsclose (xls, varargs)
         force = 1;
 
       ## Interface-specific clauses here:
-      elseif (~isempty (strfind (tolower (varargin{ii}), '.')))
+      elseif (! isempty (strfind (tolower (varargin{ii}), ".")))
         ## Apparently a file name. First some checks....
         if (xls.changed == 0 || xls.changed > 2)
           warning ("xlsclose: file %s wasn't changed, new filename ignored.", xls.filename);
         elseif (strcmp (xls.xtype, "JXL"))
           error ("xlsclose: JXL doesn't support changing filename, new filename ignored.");
-        elseif (~((strcmp (xls.xtype, "COM") || strcmp (xls.xtype, "UNO")) ... 
+        elseif (! ((strcmp (xls.xtype, "COM") || strcmp (xls.xtype, "UNO")) ... 
                 && isempty (strfind ( lower (filename), ".xls"))))
           # Excel/ActiveX && OOo (UNO bridge) will write any valid filetype; POI/JXL/OXS need .xls[x]
           error ("xlsclose: .xls or .xlsx suffix lacking in filename %s", filename);
@@ -130,12 +139,15 @@ function [ xls ] = xlsclose (xls, varargs)
   elseif (strcmp (xls.xtype, "UNO"))
     xls = __UNO_spsh_close__ (xls, force);
 
+  elseif (strcmp (xls.xtype, "OCT"))
+    xls = __OCT_spsh_close__ (xls);
+
   ## elseif   <other interfaces here>
 
   endif
 
   if (xls.changed && xls.changed < 3)
-    warning (sprintf ("xlsclose: file %s could not be saved. Read-only or in use elsewhere?",...
+    warning (sprintf ("xlsclose: file %s could not be saved. Read-only or in use elsewhere?", ...
                       xls.filename));
     if (force)
       xls = [];
