@@ -1,4 +1,4 @@
-## Copyright (C) 2009,2010,2011,2012 Philip Nienhuis <pr.nienhuis at users.sf.net>
+## Copyright (C) 2009,2010,2011,2012,2013 Philip Nienhuis <pr.nienhuis at users.sf.net>
 ##
 ## This program is free software; you can redistribute it and/or modify it under
 ## the terms of the GNU General Public License as published by the Free Software
@@ -88,19 +88,34 @@
 ## 2012-02-25 Added info on occupied ranges to sh_names outarg for all interfaces
 ## 2012-10-12 Moved all interface-specific code into ./private subfuncs
 ## 2012-10-24 Style fixes
+## 2013-10-01 OCT interface added for gnumeric
+## 2013-12-01 Style fixes & support for more file types than just Excel
+## 2013-12-29 Style fixes
 
 function [ filetype, sh_names, fformat ] = xlsfinfo (filename, reqintf=[])
 
-  persistent str2; str2 = "                                 "; # 33 spaces
+  persistent str2; str2 = "                                 "; ## 33 spaces
   persistent lstr2; lstr2 = length (str2);
 
   xls = xlsopen (filename, 0, reqintf);
-  if (isempty (xls)); return; endif
+  if (isempty (xls))
+    return; 
+  endif
 
   toscreen = nargout < 1;
 
-  ## If any valid xls-pointer struct has been returned, it must be a valid Excel spreadsheet
-  filetype = "Microsoft Excel Spreadsheet"; 
+  ## If any valid xls-pointer struct has been returned, it must be a valid
+  ## spreadsheet. Find out what format
+  [~, ~, ext] = fileparts (xls.filename); 
+  switch ext
+    case {"xls", "xlsx", "xlsm", "xlsb"}
+      filetype = "Microsoft Excel Spreadsheet";
+    case "ods"
+      filetype = "OpenOffice.org Calc spreadsheet";
+    case "gnumeric"
+      filetype = "Gnumeric spreadsheet";
+    otherwise
+  endswitch
   fformat = "";
 
   if (strcmp (xls.xtype, "COM"))
@@ -118,6 +133,9 @@ function [ filetype, sh_names, fformat ] = xlsfinfo (filename, reqintf=[])
   elseif (strcmp (xls.xtype, "UNO"))
     [sh_names] = __UNO_spsh_info__ (xls);
 
+  elseif (strcmp (xls.xtype, "OCT"))
+    [sh_names] = __OCT_spsh_info__ (xls);
+
 ##elseif   <Other Excel interfaces below>
 
   else
