@@ -1,4 +1,4 @@
-## Copyright (C) 2009,2010,2011,2012 by Philip Nienhuis <prnienhuis at users.sf.net>
+## Copyright (C) 2009,2010,2011,2012,2013 by Philip Nienhuis <prnienhuis at users.sf.net>
 ##
 ## This program is free software; you can redistribute it and/or modify it under
 ## the terms of the GNU General Public License as published by the Free Software
@@ -66,14 +66,15 @@
 ##
 ## The optional last argument @var{reqintf} can be used to override 
 ## the automatic interface selection by xlsread out of the supported
-## ones: COM/Excel, Java/Apache POI, Java/JExcelAPI, Java/OpenXLS, or 
-## Java/UNO (OpenOffice.org) (in that -built in- order of preference).
-## For reading from OOXML files a value of 'com', 'poi' or 'uno' must
-## be specified for @var{reqintf} (see help for xlsopen); for Excel'95
-## files use 'com', or if Excel is not installed use 'jxl', 'basic'
-## or 'uno' (POI can't read Excel 95 but will try to fall back to JXL).
-## As @var{reqintf} can also be a cell array of strings, one can
-## select or exclude one or more interfaces.
+## ones: COM/Excel, Java/Apache POI, Java/JExcelAPI, Java/OpenXLS, 
+## Java/UNO (OpenOffice.org), or native Octave (only reading .xlsx)
+## (in that -built in- order of preference).
+## For reading from OOXML files a value of 'com', 'poi', 'uno', or 'oct'
+## must be specified for @var{reqintf} (see help for xlsopen); for
+## Excel'95 files use 'com', or if Excel is not installed use 'jxl',
+## 'basic' or 'uno' (POI can't read Excel 95 but will try to fall back
+## to JXL). As @var{reqintf} can also be a cell array of strings, one
+## can select or exclude one or more interfaces.
 ##
 ## Erroneous data and empty cells are set to NaN in @var{numarr} and
 ## turn up empty in @var{txtarr} and @var{rawarr}. Date/time values in
@@ -91,11 +92,11 @@
 ## "corresponding" Excel cells are empty.
 ##
 ## xlsread is just a wrapper for a collection of scripts that find out
-## the interface to be used (COM, Java/POI, Java/JXL Java/OXS, Java/UNO)
-## and do the actual reading. For each call to xlsread the interface must
-## be started and the Excel file read into memory. When reading multiple
-## ranges (in optionally multiple worksheets) a significant speed boost
-## can be obtained by invoking those scripts directly as in:
+## the interface to be used (COM, Java/POI, Java/JXL Java/OXS, Java/UNO,
+## OCT) and do the actual reading. For each call to xlsread the interface
+## must be started and the Excel file read into memory. When reading
+## multiple ranges (in optionally multiple worksheets) a significant speed
+## boost can be obtained by invoking those scripts directly as in:
 ## xlsopen / xls2oct [/ parsecell] / ... / xlsclose
 ##
 ## Beware: when using the COM interface, hidden Excel invocations may be
@@ -145,18 +146,23 @@
 ## 2012-01-26 Fixed "seealso" help string
 ## 2012-03-07 Updated texinfo help header
 ## 2012-10-24 Style fixes
+## 2013-09-24 Drop requirement of having at least one output arg
+## 2013-09-27 Check for proper filename in input
+## 2013-09-27 Proper spelling of input arg
+## 2013-09-30 Header adapted to native OCT interface f xlsx
+## 2013-12-20 Style fixes
+## 2013-12-27 In case of .csv fall back to csvread for lazy Matlab users
 
 function [ numarr, txtarr, rawarr, lims ] = xlsread (fn, wsh, datrange, reqintf=[])
 
 	rstatus = 0;
 
-	if (nargout < 1)
-		usage ("xlsread: no output argument(s) specified that should receive data");
-	endif
 	if (nargin < 1) 
 		error ("xlsread: no input arguments specified") 
 		numarr = []; txtarr={}; rawarr = {};
 		return
+  elseif (! ischar (fn))
+    error ("filename (text string) expected for argument #1, not a %s", class (fn));
 	elseif (nargin == 1)
 		wsh = 1;
 		datrange = ""; 
@@ -173,7 +179,7 @@ function [ numarr, txtarr, rawarr, lims ] = xlsread (fn, wsh, datrange, reqintf=
 	endif
 
 	## A small gesture for Matlab compatibility. JExcelAPI supports BIFF5.
-	if (~isempty (reqintf) && ischar (reqintf) && strcmpi (reqintf, "BASIC")) 
+	if (! isempty (reqintf) && ischar (reqintf) && strcmpi (reqintf, "BASIC")) 
 		reqintf = {"JXL"}; 
 		printf ("(BASIC (BIFF5) support request translated to JXL)\n");
 	endif
@@ -182,33 +188,38 @@ function [ numarr, txtarr, rawarr, lims ] = xlsread (fn, wsh, datrange, reqintf=
 	## what interface to use. If none found, just return as xlsopen will complain enough
 
 	unwind_protect	# Needed to catch COM errors & able to close stray Excel invocations
-	## Get pointer array to Excel file
-	xls_ok = 0;
-	xls = xlsopen (fn, 0, reqintf);
-	if (~isempty (xls))
-		xls_ok = 1;
-	else
-		return
-	endif
+    ## Get pointer array to Excel file
+    xls_ok = 0;
+    xls = xlsopen (fn, 0, reqintf);
+    if (! isempty (xls))
+      xls_ok = 1;
+    else
+      ## Convenience for scripts from lazy Matlab users (see bug #40993):
+      [~, ~, ext] = fileparts (fn);
+      if strcmpi (fn, ".csv")
+        numarr = csvread (fn, 1, datrange);
+      endif
+      return
+    endif
 
-	## Get data from Excel file & return handle
-	[rawarr, xls, rstatus] = xls2oct (xls, wsh, datrange);
+    ## Get data from Excel file & return handle
+    [rawarr, xls, rstatus] = xls2oct (xls, wsh, datrange);
 
-	## Save some results before xls is wiped
-	rawlimits = xls.limits;
-	xtype = xls.xtype;
+    ## Save some results before xls is wiped
+    rawlimits = xls.limits;
+    xtype = xls.xtype;
 
-	if (rstatus)
-		[numarr, txtarr, lims] = parsecell (rawarr, rawlimits);
-	else
-		rawarr = {}; numarr = []; txtarr = {};
-	endif
+    if (rstatus)
+      [numarr, txtarr, lims] = parsecell (rawarr, rawlimits);
+    else
+      rawarr = {}; numarr = []; txtarr = {};
+    endif
 
 	unwind_protect_cleanup	
-	## Close Excel file
-	if (xls_ok)
-    xls = xlsclose (xls);
-  endif
+    ## Close Excel file
+    if (xls_ok)
+      xls = xlsclose (xls);
+    endif
 
 	end_unwind_protect
 
